
package dataSetDescribers;

import java.util.HashSet;
import java.util.Iterator;

public abstract class AbstractSpikedControls
{
    private HashSet<String> allGeneIDs = null;
    
    // 0 may be null here to keep the numbering the same as in Affy Experiments
    abstract public HashSet<String>[] getGeneIDs() throws Exception;
    
    abstract public boolean isExcludedProbeSet(String probeSetId) throws Exception;
    
    abstract public float[][] getConcentrations() throws Exception;
    
    
    public int getSpikeRatio(int baselineSpikeNumber, int expSpikeNumber) throws Exception
    {
    	int spikeRatio = -1;
    	
    	float[] first = getConcentrations()[baselineSpikeNumber];
    	float[] second = getConcentrations()[expSpikeNumber];
    	
    	if( first.length != second.length ) 
    		throw new Exception("Logic error");
    	
    	for ( int x=0; x < first.length; x++   )
    		if( first[x] > 0.000000001 && second[x] > 0.000000001 )
			{
				//System.out.println( first[x] + " " + second[x] );
				float ratio = first[x] / second[x];
				
				if( ratio < 1  )
					ratio = 1 / ratio;
				
				int newRatio = (int) (ratio + .1);
				
				if( spikeRatio == -1 )
					spikeRatio = newRatio;
				
				if( spikeRatio != newRatio )
					System.out.println("Found two different ratios " + spikeRatio +  " " + newRatio + 
							" for " + baselineSpikeNumber + "@" + expSpikeNumber );
 			}
    		
    	
    	return spikeRatio;
    }
    
    public float getAverageDifferenceInSpikeConcentration(int baselineSpikeNumber, int expSpikeNumber) throws Exception
    {
    	float sum=0;
    	int n=0;
    	
    	float[] first = getConcentrations()[baselineSpikeNumber];
    	float[] second = getConcentrations()[expSpikeNumber];
    	
    	if( first.length != second.length ) 
    		throw new Exception("Logic error");
    	
    	for ( int x=0; x < first.length; x++   )
    		if( first[x] > 0.000000001 && second[x] > 0.000000001 )
			{
    			sum+= Math.abs( first[x] - second[x] );
    			n++;
    		}
    		
    	
    	return sum /n;
    }
    
    /** 
     *   Throws if the groupID can't be found
     */
    @SuppressWarnings("unchecked")
	public int getGroupId(String probeSet) throws Exception
    {
        HashSet[] geneIds = getGeneIDs();
        
        for ( int x=0; x < geneIds.length; x++ )
        {
            if (geneIds[x] != null)
            {
                for (Iterator i = geneIds[x].iterator(); i.hasNext(); )
                {
                    if( i.next().equals(probeSet) )
                        return x;
                }
            }
        }
        
        throw new Exception("Could not find " + probeSet);
    }
    
    /**
     *   LAZY INITIALIZED AND NOT THREAD SAFE
     */
    @SuppressWarnings("unchecked")
	public HashSet<String> getAllSpikedGeneIds() throws Exception
    {
        if ( allGeneIDs == null)
        {
            HashSet<String> set = new HashSet<String>();
            
            HashSet[] allGenes = getGeneIDs();
            
            for ( int x=0; x < allGenes.length; x++  )
            {
                if ( allGenes[x] != null)
                {
                    for ( Iterator i = allGenes[x].iterator(); i.hasNext(); )
                    {
                        set.add(i.next().toString());
                    }
                }
            }
            
            allGeneIDs = set;
        }
        
        return allGeneIDs;
    }
}
