package dataSetDescribers;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.tigr.midas.engine.Lowess;

import statistics.StatisticsLevelNormalization;

public class ExperimentData
{
	private List<ProbeSetData> probeSetData = new ArrayList<ProbeSetData>();
	private Double variance = null; 
	
	public void writeHeaderlessDataToFile(File file,boolean subtractLowess) throws Exception
	{
		float[] yEst = null;
		
		if( subtractLowess)
			yEst = getLoessNormalizationCorrection();
		
		BufferedWriter writer = new BufferedWriter(new FileWriter(new File( 
				file.getAbsolutePath()  )));
		
		for( int x = 0; x < probeSetData.size(); x++ )
		{
			ProbeSetData data = probeSetData.get(x);
			
			for( Float f : data.getBaselineData())
			{
				float dataPoint = f;
				
				if( subtractLowess)
					dataPoint += yEst[x]/2;
				
				writer.write(dataPoint + "\t");
			}
			
			
			for(  Iterator<Float> i = data.getExperimentData().iterator(); i.hasNext();)
			{
				Float dataPoint = (Float) i.next();
				
				if( subtractLowess)
					dataPoint -= yEst[x]/2;
				
				writer.write(dataPoint + ( i.hasNext() ? "\t" : "\n" )  );
			}
				
		}
		
		writer.flush();  writer.close();
		
	}
	
	public float[] getLoessNormalizationCorrection() throws Exception
	{
		float[] x = new float[probeSetData.size()];
		float[] y = new float[probeSetData.size()];
	
		//BufferedWriter writer = new BufferedWriter(new FileWriter(new File("c:\\temp\\quickOut.txt")));
		
		for(int i = 0; i < x.length; i++)
		{
			ProbeSetData psd = probeSetData.get(i);
			psd.setSortableStorage( psd.getBaselineAverage() + psd.getExperimentAverage());
		}
		
		Collections.sort(probeSetData);
		
		for(int i = 0; i < x.length; i++)
		{
			ProbeSetData psd = probeSetData.get(i);
			x[i] =  psd.getBaselineAverage() + psd.getExperimentAverage();
			y[i] = psd.getExperimentAverage() - psd.getBaselineAverage();
			//writer.write(i + "\t" + x[i] + "\t" + y[i] + "\n");
		}
		//writer.flush();  writer.close();
		
		
		Lowess lowess = new Lowess(x,y,StatisticsLevelNormalization.WINDOW_SIZE);
		return lowess.getYEst();
	} 
	
	public int getNumberOfExperiments() throws Exception
	{
		int numExperiments = -1;
		
		for( ProbeSetData data : probeSetData )
		{
			if( numExperiments == -1)
				numExperiments = data.getExperimentData().size();
			
			if( numExperiments  != data.getExperimentData().size() )
				throw new Exception("Unever number of experiments ");
		}
		
		return numExperiments;
	}
	

	public int getNumberOfBaseline() throws Exception
	{
		int numBaseline = -1;
		
		for( ProbeSetData data : probeSetData )
		{
			if( numBaseline == -1)
				numBaseline = data.getBaselineData().size();
			
			if( numBaseline  != data.getBaselineData().size() )
				throw new Exception("Unever number of experiments ");
		}
		
		return numBaseline;
	}
	
	public HashMap<String, ProbeSetData> getAsMap() throws Exception
	{
		HashMap<String, ProbeSetData> map = new HashMap<String, ProbeSetData>();
		
		for( ProbeSetData ps : probeSetData )
		{
			if( map.get(ps.getProbeSetId()) != null)
				throw new Exception("Duplicate probe " + ps.getProbeSetId());
			
			map.put( ps.getProbeSetId(), ps );
				
		}	
		return map;
	}
	
	public List<ProbeSetData> getProbeSetData()
	{
		return probeSetData;
	}

	public void setProbeSetData(List<ProbeSetData> probeSetData)
	{
		this.probeSetData = probeSetData;
	}

	/*
	 *   Not thread safe
	 */
	public void addProbeSetData( ProbeSetData data )
	{
		this.probeSetData.add(data);
		variance = null;
	}
	
	/*
	 *   Not thread sae
	 */
	public double getAverageVaraiance() throws Exception
	{
		if ( variance == null)
		{
			double sum =0;
			int n = 0;
			
			for( ProbeSetData data : probeSetData )
			{
				sum += data.getPooledVariance();
				n++;
			}
			
			variance = sum / n;
		}
		
		return variance;
	}
}
