package dataSetDescribers;

import java.io.File;

import statistics.StatisticsProvider;
import utils.ConfigReader;

public class FilePathGenerator
{
	/*
	 *   Static methods only
	 */
	private FilePathGenerator()
	{
		
	}
	
	public static File deleteAndEnsureDeleted( String filePath ) throws Exception
	{
		File file = new File(filePath);
		
		file.delete();
		
		if ( file.exists())
			throw new Exception( "Could not delete " + filePath );
		
		return file;
	}
	
	/*
	 *   R needs forward slashes no matter what platform you are on.
	 */
	public static String getRPath( File inPath ) throws Exception
	{
		String inString = inPath.getAbsolutePath();
		
		StringBuffer buff= new StringBuffer();
		
		for ( int x=0; x < inString.length(); x++)
		{
			char c= inString.charAt(x);
			
			if ( c== '\\') 
				buff.append("/");
			else
				buff.append(c);
		}
		
		return buff.toString();
	}
	
	public static File getAndMakeFDRDir( RatioExperiment rei ) throws Exception
	{
		File dir = new File(  ConfigReader.getFdrAnalysisOutputDir() + File.separator + 
				rei.getShortExperimentName());
		
		if( ! dir.exists())
			dir.mkdir();
		
		return dir;
	}
	
	public static File getAndMakeExperimentDir(RatioExperiment rei  ) throws Exception
	{
		File dir = new File(  ConfigReader.getAffyAnalysisOutputDir() + File.separator + 
				rei.getShortExperimentName());
		
		if( ! dir.exists())
			dir.mkdir();
		
		return dir;
	}
	
	public static File getAdjustedPValueScores( RatioExperiment rei, StatisticsProvider sp  ) 
	throws Exception
	{
		return new File(getAndMakeExperimentDir(rei) + File.separator + rei.getExperimentDescripton() 
				+  "_" + sp.getDescription() +"_adjustedPs.txt" );
	}
	
	public static File getScoreFile( RatioExperiment rei, StatisticsProvider sp ) throws Exception
	{
		String firstPart = 
			getAndMakeExperimentDir(rei) + File.separator + rei.getExperimentDescripton() 
			+  "_" + sp.getDescription();
		
		if( firstPart.endsWith("_"))
			firstPart = firstPart.substring(0, firstPart.length() -1);
		
		return new File( firstPart +".jitn" );
	}
	
	public static File getROCFile( RatioExperiment rei, StatisticsProvider sp )
		throws Exception
	{
		return new File(getAndMakeExperimentDir(rei) + File.separator + rei.getExperimentDescripton() 
				+  "_" + sp.getDescription() +"_ROC.txt" );
	}
	
	public static File getROCFileNoFractions( RatioExperiment rei, StatisticsProvider sp )
	throws Exception
	{
		return new File(getAndMakeExperimentDir(rei) + File.separator + rei.getExperimentDescripton() 
			+  "_" + sp.getDescription() +"_ROCNoFractions.txt" );
	}
	
	public static File getExpectedVsActualFalsePositiveFile( RatioExperiment rei, 
																	StatisticsProvider sp )
		throws Exception
	{
		return new File(getAndMakeExperimentDir(rei) + File.separator + rei.getExperimentDescripton() 
				+  "_" + sp.getDescription() +"_ExpectedVsActualFalsePositive.txt" );
	}
	
	public static File getFDRFile( RatioExperiment re, 
								   StatisticsProvider sp, 
								   boolean replaceWithNormal  ) throws Exception
	{
		return new File( getAndMakeFDRDir(re) 
				+ File.separator + re.getExperimentDescripton() 
				+  "_" + sp.getDescription() + "_replaceWithNormal" + replaceWithNormal + 
				"_FDRs.txt" );
	}
	

	public static File getFDRFile( RatioExperiment re, 
								   StatisticsProvider sp) throws Exception
	{
		return new File( getAndMakeFDRDir(re) 
				+ File.separator + re.getExperimentDescripton() 
				+  "_" + sp.getDescription() + "_FDRs.txt" );
	}
}
