
package dataSetDescribers;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.HashSet;
import java.util.Iterator;

/**
 *   This class takes the excel file HG-U133A_tag_Latin_Square.xls
 *   downloaded from http://www.affymetrix.com/support/technical/sample_data/datasets.affx
 *   and just makes it Java accessible
 */
public class HGU133ALatinSquare extends AbstractSpikedControls
{
    private float[][] concentrations = null;
    private HashSet<String>[] geneIds = null;
    
    
    /**
     *   0 is undefined for both x and y
     *   the x is the experiment number
     *   the y is the group id.
     * 
     *   LAZY INITIALIZED AND NOT THREAD SAFE!
     */
    public float[][] getConcentrations()
    {
        if ( concentrations == null)
        {
            float[][] a = new float[15][15];
	        
	        int topVal = 512000;
	        
	        for ( int x=14; x>=1; x--)
	        {
	            int bottomVal = topVal;
	            
	            for (int y=1; y<15; y++)
	            {
	                a[x][y] = bottomVal / 1000f;
	                
	                if ( bottomVal == 0 )
	                    bottomVal = 125;
	                else
	                    bottomVal = bottomVal * 2;
	                
	                if ( bottomVal > 512000)
	                    bottomVal = 0;
	            }
	            
	            topVal /= 2;
	            if (topVal < 125)
	                topVal = 0;
	        }
	        
	        concentrations = a;
        }
        
        return concentrations;
    }
    
    /**
     *   the 0th element is null
     *   1 - 14 match Affy group Ids
     * 
     *   LAZY INITIALIZED AND NOT THREAD SAFE!
     */
    @SuppressWarnings("unchecked")
	public HashSet<String>[] getGeneIDs()
    {
        if ( geneIds == null)
        {
            HashSet<String>[] sets = new HashSet[15];
	        sets[0] = null; 
	        
	        sets[1] = new HashSet<String>();
	        sets[1].add("203508_at");  sets[1].add("204563_at");  sets[1].add("204513_s_at");
	        
	        sets[2] = new HashSet<String>();
	        sets[2].add( "204205_at");  sets[2].add("204959_at");  sets[2].add("207655_s_at");
	        
	        sets[3] = new HashSet<String>();
	        sets[3].add( "204836_at");  sets[3].add("205291_at");  sets[3].add("209795_at");
	        
	        sets[4] = new HashSet<String>();
	        sets[4].add("207777_s_at");  sets[4].add("204912_at");  sets[4].add("205569_at");
	        
	        sets[5] = new HashSet<String>();
	        sets[5].add("207160_at");  sets[5].add("205692_s_at");  sets[5].add("212827_at");
	        
	        sets[6] = new HashSet<String>();
	        sets[6].add("209606_at");  sets[6].add("205267_at");  sets[6].add("204417_at");
	        
	        sets[7] = new HashSet<String>();
	        sets[7].add("205398_s_at");  sets[7].add("209734_at");  sets[7].add("209354_at");
	        
	        sets[8] = new HashSet<String>();
	        sets[8].add("206060_s_at");  sets[8].add("205790_at");  sets[8].add("200665_s_at");
	        
	        sets[9] = new HashSet<String>();
	        sets[9].add("207641_at");  sets[9].add("207540_s_at");  sets[9].add("204430_s_at");
	        
	        sets[10] = new HashSet<String>();
	        sets[10].add("203471_s_at");  sets[10].add("204951_at");  sets[10].add("207968_s_at");
	        
	        sets[11] = new HashSet<String>();
	        sets[11].add("AFFX-r2-TagA_at");  sets[11].add("AFFX-r2-TagB_at");  sets[11].add("AFFX-r2-TagC_at");
	        
	        sets[12] = new HashSet<String>();
	        sets[12].add("AFFX-r2-TagD_at");  sets[12].add("AFFX-r2-TagE_at");  sets[12].add("AFFX-r2-TagF_at");
	        
	        sets[13] = new HashSet<String>();
	        sets[13].add("AFFX-r2-TagG_at");  sets[13].add("AFFX-r2-TagH_at");  sets[13].add("AFFX-DapX-3_at");
	        
	        sets[14] = new HashSet<String>();
	        sets[14].add("AFFX-LysX-3_at");  sets[14].add("AFFX-PheX-3_at"); sets[14].add("AFFX-ThrX-3_at");
	        
	        geneIds = sets;
        }
        
        return geneIds;
    };
    
    /*
     * From the Latin Square documentation:
     * 
     * The following additional probe sets are known to contain 
     * probes that exactly match the spiked clone sequences 
     * (due to placement of multiple probe sets for 
     * alternative polyadenylation sites and/or identical 
     * 3' regions among paralogs)
     * 
     * In addition, we here exclude any probe that starts with AFFX
     * as these may be control probes ( need to find out more about that)
     * 
     * LAZY INITIALIZED AND NOT THREAD SAFE!
     * 
     */
    public boolean isExcludedProbeSet(String probeId) throws Exception
    {
    	if ( probeId.equals("209374_s_at"))
    		return true;
    		
    	if ( probeId.equals("205397_x_at"))
    		return true;
    	
    	if ( probeId.equals("208010_s_at"))
    		return true;
    	
    	if( this.getAllSpikedGeneIds().contains( probeId ) )
    		return false;
    	
    	if ( probeId.toUpperCase().startsWith("AFFX") )
    		return true;
    	
    	return false;
    }
    
    public static void main(String[] args) throws Exception
    {
        BufferedWriter writer =new BufferedWriter(new FileWriter(new File("/Users/afodor/temp/mySquares.txt")));
        
        HGU133ALatinSquare ls = new HGU133ALatinSquare();
        
        HashSet<String>[] geneIds = ls.getGeneIDs();
        
        writer.write("\t");
        for ( int x=1; x < geneIds.length; x++ )
        {
            for (Iterator<String> i = geneIds[x].iterator(); i.hasNext(); )
            {
                writer.write(i.next().toString());
                
                if ( i.hasNext())
                    writer.write("&");
                else if (x < geneIds.length + 1 )
                    writer.write("\t");
            }
        }
        
        writer.write("\n");
        
        float[][] concentrations = ls.getConcentrations();
        
        for ( int x=1; x < concentrations.length; x++ ) 
        {
            writer.write(x + "\t");
            
            for ( int y=1; y < concentrations[x].length; y++)
            {
                writer.write(concentrations[x][y] + "\t");
            }
            
            writer.write("\n");
        }
        
        writer.flush();  writer.close();
    }
}
