package dataSetDescribers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;


import utils.ConfigReader;

public class LatinSquareGeneric extends RatioExperiment
{
	protected int baselineNum;
	protected int experimentNum;
	
	public List<String> experimentList  = null;
	public List<String> baselineList = null;
	
	private AbstractSpikedControls spikeDetails = null;
    
	public LatinSquareGeneric(int baselineNum, int experimentNum)
	{
		this.baselineNum = baselineNum;
		this.experimentNum = experimentNum;
	}
	
    @Override
    public File getCdfFile() throws Exception
    {
        return new File(ConfigReader.getLatinSquareDirectory() + File.separator +
                "HG-U133A_tag.CDF");
    }
	
    @Override
    public String getChipId()
    {
        return "HG-U133A_tag";
    }
	    
    public static List<LatinSquareGeneric> getAll91()
    {
    	List<LatinSquareGeneric> list = new ArrayList<LatinSquareGeneric>();
    	
    	for( int x =1; x <= 13; x++ )
			for(int y=x+1; y <= 14;y++)	
			{
				LatinSquareGeneric re = new LatinSquareGeneric(x,y);
				list.add(re);
			}
    	
    	return list;
    }

    public static List<RatioExperiment> getAll2X()
    {
    	int[] x = {1,2,3,4,5,6,7,8,9,10,11,12,13,1};
		int[] y = {2,3,4,5,6,7,8,9,10,11,12,13,14,14};
		
    	
    	List<RatioExperiment> list = new ArrayList<RatioExperiment>();
    	
    	for( int i=0; i < x.length; i++ )
    	{
			LatinSquareGeneric re = new LatinSquareGeneric(x[i],y[i]);
			list.add(re);
		}
	
    	return list;
    }
    

	@Override
	public File getCelFileDirectory() throws Exception
	{
		return new File(ConfigReader.getLatinSquareDirectory());
	}

	@Override
	public void setBaselineFiles(List<String> list) throws Exception
	{
		this.baselineList = list;
	}
	
	@Override
	public void setExperimentFiles(List<String> list) throws Exception
	{
		this.experimentList = list;
	}
	
	/*
	 * Not thread safe
	 */
	public List<String> getBaselineFiles() throws Exception
	{
		if(baselineList == null)
		{
			baselineList = getFileNames(baselineNum);
		}
		
		return baselineList;
	}

	/*
	 *  Not thread safe
	 */
	public List<String> getExperimentFiles() throws Exception
	{
		if( experimentList == null)
		{
			experimentList = getFileNames(experimentNum);
		}
		
		return experimentList;
	}
	
	
	public File getNonQQRmaGepFilePath() throws Exception
	{
		return new 
		File( ConfigReader.getLatinSquareDirectory() + File.separator + 
			"RMA_LATIN_SQUARE_NOT_NORMALIZED.txt" );
	}

	@Override
	public File getQQRmaFilePath() throws Exception
	{
		return new 
		File( ConfigReader.getLatinSquareDirectory() + File.separator + 
		"RMA_LATIN_SQUARE_RESULTS.txt" );
	}

	@Override
	public File getRandomRmaGepFilePath() throws Exception
	{
		return new 
		File( ConfigReader.getLatinSquareDirectory() + File.separator + 
			"RMA_RANDOM_LATIN_SQUARE_DATA.txt");
	}

	@Override
	public String getShortExperimentName()
	{
		return "LS" + baselineNum + "@" + experimentNum;
	}

	private List<String> getFileNames(int num ) throws Exception
    {
    	List<String> list = new ArrayList<String>();
        
        for( int x=1; x <=3; x++ )
        {
        		list.add( "12_13_02_U133A_Mer_Latin_Square_Expt" + 
        	     			num +"_R" + x + ".CEL");
        }
     
        return list;
    }

    
    /**
     *   Lazy initialized and not thread safe
     */
    public AbstractSpikedControls getSpikeDetails()
    {
        if ( spikeDetails == null)
        {
            spikeDetails = new HGU133ALatinSquare();
        }
        
        return spikeDetails;
    }
    
	public int getBaselineNum()
	{
		return baselineNum;
	}

	public void setBaselineNum(int baselineNum)
	{
		this.baselineNum = baselineNum;
	}

	public int getExperimentNum()
	{
		return experimentNum;
	}

	public void setExperimentNum(int experimentNum)
	{
		this.experimentNum = experimentNum;
	}
	
	public static List<RatioExperiment> get8VS9()
    {
    	int[] x = {8};
		int[] y = {9};
		
    	
    	List<RatioExperiment> list = new ArrayList<RatioExperiment>();
    	
    	for( int i=0; i < x.length; i++ )
    	{
			LatinSquareGeneric re = new LatinSquareGeneric(x[i],y[i]);
			list.add(re);
		}
	
    	return list;
    }
	
}
