package dataSetDescribers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import utils.ConfigReader;

public class LatinSquareGenericProbe extends ProbeAbstract
{		
	private AbstractSpikedControls spikeDetails = null;
	private final List<File> baselineCelFiles;
	private final List<File> experimentCelFiles;

    @Override
    public File getCdfFile() throws Exception
    {
    	File cdfFile = new File( ConfigReader.getCDFFile() );
    	
    	if( ! cdfFile.exists())
    		throw new Exception("Could not find cdf file " + cdfFile.getAbsolutePath());
    	
    	return cdfFile;
    }
    
    @Override
    public String getChipId() throws Exception
    {
    	File cdfFile = getCdfFile();
    	
    	String returnVal = cdfFile.getName();
    
    	if( returnVal.toUpperCase().endsWith(".CDF"))
    		returnVal = returnVal.substring(0, returnVal.length()-4);
    	
        return ConfigReader.getCDFFile();
    }
		
	public LatinSquareGenericProbe( List<File> baselineCelFiles, 
			List<File> experimentCelFiles ) 
	{
		this.baselineCelFiles = baselineCelFiles;
		this.experimentCelFiles = experimentCelFiles;
	}
	
	private static File getDChipQQNormFile(int exp, int rep ) throws Exception
	{
		return new File(
		ConfigReader.getLatinSquareDirectory() + File.separator +
		"dChipQQNorm" + File.separator + 
		"12_13_02_U133A_Mer_Latin_Square_Expt" + exp + "_R" + rep + ".cel" );
		
	}
	
	public static List<LatinSquareGenericProbe> getAll2XRep1VsRep1() throws Exception
	{
		List<LatinSquareGenericProbe> returnList = new ArrayList<LatinSquareGenericProbe>();
		
		int[] x = {1,2,3,4,5,6,7,8,9,10,11,12,13,1};
		int[] y = {2,3,4,5,6,7,8,9,10,11,12,13,14,14};
		
		
		
		for( int i = 0; i < x.length; i++)
		{
			List<File> baselineFiles = new ArrayList<File>();
			baselineFiles.add( getDChipQQNormFile(x[i],1) );
			
			List<File> experimentFiles = new ArrayList<File>();
			experimentFiles.add( getDChipQQNormFile(y[i],1) );
			
			returnList.add( new LatinSquareGenericProbe(baselineFiles, experimentFiles) );
		}
		
		return returnList;
	}
	
	public static List<LatinSquareGenericProbe> get3RandomXRep123VsRep123() throws Exception
	{
		List<LatinSquareGenericProbe> returnList = new ArrayList<LatinSquareGenericProbe>();
		
		//choosing 1 randomly...  y is then the random # plus 1. Then we need to get
		// all 3 reps from that 1.   Random nnumber was 4.....
		int[] x = {4};
		int[] y = {5};
		
		// 1 
		for( int i = 0; i < x.length; i++)
		{
			for (int j = 1; j <= 3; j++)
			{
				List<File> baselineFiles = new ArrayList<File>();
				baselineFiles.add( getDChipQQNormFile(x[i],j) );
				
				for (int k = 1; k <= 3; k++)
				{
					
					List<File> experimentFiles = new ArrayList<File>();
					experimentFiles.add( getDChipQQNormFile(y[i],k) );
					
					//returnList.add( new LatinSquareGenericProbe(baselineFiles, experimentFiles) );
					returnList.add( new LatinSquareGenericProbe(baselineFiles, experimentFiles) );
					
				}	
			}	
		}
		
		return returnList;
	}
	
	public static List<LatinSquareGenericProbe> getAll2XRep1And2VsRep1And2() throws Exception
	{
		List<LatinSquareGenericProbe> returnList = new ArrayList<LatinSquareGenericProbe>();
		
		int[] x = {1,2,3,4,5,6,7,8,9,10,11,12,13,1};
		int[] y = {2,3,4,5,6,7,8,9,10,11,12,13,14,14};
		
		
		
		for( int i = 0; i < x.length; i++)
		{
			List<File> baselineFiles = new ArrayList<File>();
			baselineFiles.add( getDChipQQNormFile(x[i],1) );
			baselineFiles.add( getDChipQQNormFile(x[i],2) );
			
			List<File> experimentFiles = new ArrayList<File>();
			experimentFiles.add( getDChipQQNormFile(y[i],1) );
			experimentFiles.add( getDChipQQNormFile(y[i],2) );
			
			returnList.add( new LatinSquareGenericProbe(baselineFiles, experimentFiles) );
		}
		
		return returnList;
	}

	public static List<LatinSquareGenericProbe> getAll2XRep1And2And3VsRep1And2And3() throws Exception
	{
		List<LatinSquareGenericProbe> returnList = new ArrayList<LatinSquareGenericProbe>();
		
		int[] x = {1,2,3,4,5,6,7,8,9,10,11,12,13,1};
		int[] y = {2,3,4,5,6,7,8,9,10,11,12,13,14,14};
		
		
		
		for( int i = 0; i < x.length; i++)
		{
			List<File> baselineFiles = new ArrayList<File>();
			baselineFiles.add( getDChipQQNormFile(x[i],1) );
			baselineFiles.add( getDChipQQNormFile(x[i],2) );
			baselineFiles.add( getDChipQQNormFile(x[i],3) );
			
			List<File> experimentFiles = new ArrayList<File>();
			experimentFiles.add( getDChipQQNormFile(y[i],1) );
			experimentFiles.add( getDChipQQNormFile(y[i],2) );
			experimentFiles.add( getDChipQQNormFile(y[i],3) );
			
			returnList.add( new LatinSquareGenericProbe(baselineFiles, experimentFiles) );
		}
		
		return returnList;
	}
	
	@Override
	public List<File> getBaselineCelFiles()
	{
		return baselineCelFiles;
	}

	@Override
	public List<File> getExperimentCelFiles()
	{
		// TODO Auto-generated method stub
		return experimentCelFiles;
	}

	@Override
	int getNumExpectedProbes()
	{
		return 11;
	}

	private String getShortName(File file)
	{
		String name = file.getName();
		//name = name.substring("12_13_02_U133A_Mer_Latin_Square_Expt".length());
		name = name.substring(0, name.indexOf("."));
		return name;
	}
	
	public static void main(String[] args) throws Exception
	{
		//List<LatinSquareGenericProbe> list = getAll2XRep1VsRep1();
		List<LatinSquareGenericProbe> list = getAveraged3RandomChosen45();
		
		for( LatinSquareGenericProbe lsg : list )
		{
			System.out.println(lsg.getShortExperimentName());
			
			for( File f : lsg.getBaselineCelFiles() )
				if( ! f.exists())
					throw new Exception(f.getAbsolutePath());
			

			for( File f : lsg.getExperimentCelFiles() )
				if( ! f.exists())
					throw new Exception(f.getAbsolutePath());
		}
			
	}
	
	@Override
	public String getShortExperimentName()
	{
		StringBuffer buff = new StringBuffer();
		
		for( File file : getBaselineCelFiles() )
			//buff.append( getShortName(file).substring(36,40) );
			buff.append( getShortName(file) );
		
		buff.append("VS");
		
		for( File file : getExperimentCelFiles() )
			//buff.append( getShortName(file).substring(36,40) );
			buff.append( getShortName(file) );
		return buff.toString();
	}
	
	 /**
     *   Lazy initialized and not thread safe
     */
    public AbstractSpikedControls getSpikeDetails()
    {
        if ( spikeDetails == null)
        {
            spikeDetails = new HGU133ALatinSquare();
        }
        
        return spikeDetails;
    }
    
    public static List<LatinSquareGenericProbe> getAveraged3RandomChosen45() throws Exception
	{
		List<LatinSquareGenericProbe> returnList = new ArrayList<LatinSquareGenericProbe>();
		
		List<String> setcontrol = new ArrayList<String>();
		setcontrol.add("Latin_Square_Expt4_combined.CEL");
						
		List<String> setExpt = new ArrayList<String>();
		setExpt.add("Latin_Square_Expt5_combined.CEL");
		
		for( int i = 0; i < setcontrol.size(); i++)
		{
			List<File> controlFiles =  new ArrayList<File>();
			controlFiles.add( new File(ConfigReader.getLatinSquareDirectory() + File.separator +
					"dChipQQNorm" + File.separator + 
					"Latin_Square_Expt4_combined.CEL" ));
			
			List<File> exptFiles =  new ArrayList<File>();
			exptFiles.add( new File(ConfigReader.getLatinSquareDirectory() + File.separator +
					"dChipQQNorm" + File.separator + 
			"Latin_Square_Expt5_combined.CEL" ));

			
			returnList.add( new LatinSquareGenericProbe(controlFiles, exptFiles) );
		}
		return returnList;
	}
    
    public static List<LatinSquareGenericProbe> getAveraged2RandomChosen45() throws Exception
	{
    	// this is a bit of a hack, change file names in 2 different spots !!!!
    	List<LatinSquareGenericProbe> returnList = new ArrayList<LatinSquareGenericProbe>();
		
		List<String> setcontrol = new ArrayList<String>();
		setcontrol.add("Latin_Square_Expt4_combined2.CEL");
		
		List<String> setExpt = new ArrayList<String>();
		setExpt.add("Latin_Square_Expt5_combined2.CEL");
		
		for( int i = 0; i < setcontrol.size(); i++)
		{
			List<File> controlFiles =  new ArrayList<File>();
			controlFiles.add( new File(ConfigReader.getLatinSquareDirectory() + 
					File.separator + "dChipQQNorm" + File.separator + 
					"Latin_Square_Expt4_combined2.CEL" ));
			
			List<File> exptFiles =  new ArrayList<File>();
			exptFiles.add( new File(ConfigReader.getLatinSquareDirectory() +
					File.separator + "dChipQQNorm" + File.separator + 
					"Latin_Square_Expt5_combined2.CEL" ));

			returnList.add( new LatinSquareGenericProbe(controlFiles, exptFiles) );
		}		
		return returnList;
	}
    
    public static List<LatinSquareGenericProbe> getAveraged3VS3() throws Exception
	{
		List<LatinSquareGenericProbe> returnList = new ArrayList<LatinSquareGenericProbe>();
		List<String> setcontrol = new ArrayList<String>();
		List<String> setExpt = new ArrayList<String>();
		
		
		for (int i = 1; i <= 13; i++ )
		{
			int exptCounter = i + 1;
			
			setcontrol.add("Latin_Square_Expt" + i + "_combined.CEL");
			setExpt.add("Latin_Square_Expt" + exptCounter + "_combined.CEL");
		}
		
		for( int i = 0; i < setcontrol.size(); i++)
		{
			List<File> controlFiles =  new ArrayList<File>();
			controlFiles.add( new File(ConfigReader.getLatinSquareDirectory() + File.separator +
					"dChipQQNorm" + File.separator + setcontrol.get(i)));
					
			List<File> exptFiles =  new ArrayList<File>();
			exptFiles.add( new File(ConfigReader.getLatinSquareDirectory() + File.separator +
					"dChipQQNorm" + File.separator + setExpt.get(i)));

			returnList.add( new LatinSquareGenericProbe(controlFiles, exptFiles) );
		}
		return returnList;
	}
		
}
