package dataSetDescribers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import parsers.GeneExperimentParser;

public abstract class ProbeAbstract extends RatioExperiment
{
	abstract int getNumExpectedProbes() throws Exception;
	
	private GeneExperimentParser gep = null;
	
	public String getBaselinePrefix() throws Exception
	{
		return "B" + getShortExperimentName();
	}
	
	public String getExperimentPrefix() throws Exception
	{
		return "E" + getShortExperimentName();
	}
	
	abstract public List<File> getBaselineCelFiles();
	abstract public List<File> getExperimentCelFiles();
	
	@Override
	public boolean isLogBase2()
	{	
		return true;
	}
	
	
		@Override
	public void setUseQuantileQuantileNormalization(
			boolean useQuantileQuantileNormalization) 
	{
		if( ! useQuantileQuantileNormalization)
			throw new RuntimeException("Only supports normalized data");
		
		super.setUseQuantileQuantileNormalization(useQuantileQuantileNormalization);
	}
	
	
	@Override	
	public void setUseRandomRmaData(boolean useRandomRmaData)
	{
		if( useRandomRmaData)
			throw new RuntimeException("Does not support randomized data");
		
		super.setUseRandomRmaData(useRandomRmaData);
	}
	
	
	@Override
	public void setExperimentFiles(List<String> list) throws Exception
	{
		throw new Exception(this.getClass().getName() 
				+ " does not support a mutable experimentDescription list");
	}
	
	@Override
	public void setBaselineFiles(List<String> list) throws Exception
	{
		throw new Exception(this.getClass().getName() 
				+ " does not support a mutable baselineDescription list");
	}
	

	
	@Override
	public List<String> getBaselineFiles() throws Exception
	{
		List<String> list = new ArrayList<String>();
		
		for( int x=0; x < getNumExpectedProbes(); x++)
			list.add( getBaselinePrefix() + x);
			
		return list;
		
	}

	@Override
	public List<String> getExperimentFiles() throws Exception
	{
		List<String> list = new ArrayList<String>();
		
		for( int x=0; x < getNumExpectedProbes(); x++)
			list.add( getExperimentPrefix() + x);
			
		return list;
		
	}

	protected GeneExperimentParser getRMAResults(List<File> baseFiles, 
					List<File> expFiles, String baselinePrefix, String experimentPrefix) throws Exception
	{
		if( gep == null)
    	{
    		gep = new GeneExperimentParser( getCdfFile(), getChipId(), 
    			    baseFiles, expFiles, getNumExpectedProbes(), baselinePrefix, experimentPrefix);
    	}
    	
    	return gep;
	}
	
	public void clearRMACache() throws Exception
	{
		this.gep = null;
		super.clearRMACache();
	}
	
	public File getNonQQRmaGepFilePath() throws Exception
	{
		return null;
	}

	@Override
	public File getQQRmaFilePath() throws Exception
	{
		return null;
	}

	@Override
	public File getRandomRmaGepFilePath() throws Exception
	{
		return null;
	}
	
	@Override
	public AbstractSpikedControls getSpikeDetails()
	{
		return null;
	}

	@Override
	public GeneExperimentParser getRMAResults() throws Exception
	{
		return getRMAResults(  getBaselineCelFiles(), getExperimentCelFiles(),
				getBaselinePrefix(), getExperimentPrefix());
	}	
}
