package dataSetDescribers;

import java.io.File;
import java.util.List;

import parsers.GeneExperimentParser;

/*
 *   WARNING:  This caches the RMA data, so if you switch to an experiment with reliance on a different
 *   set of RMA files, you must call clearRMACache() to force the new data into RAM.
 */
public abstract class RatioExperiment
{
	private boolean useRandomRmaData=false;
	private boolean useQuantileQuantileNormalization=false;
	private boolean useStatisticLevelNormalization=false;
	private int shuffleAcrossAllDataNum = 0;
	
	private static GeneExperimentParser randomRmaGep = null;
	private static GeneExperimentParser qqRmaGep = null;
	private static GeneExperimentParser nonQQRmaGep = null;
	
	private DataScope dataScope = DataScope.ALL;
	
	public abstract File getRandomRmaGepFilePath() throws Exception;
	public abstract File getQQRmaFilePath() throws Exception;
	public abstract File getNonQQRmaGepFilePath() throws Exception;
	
	public void setShuffleAcrossAllDataNum(int shuffleAcrossAllDataNum)
	{
		this.shuffleAcrossAllDataNum = shuffleAcrossAllDataNum;
	}
	
	public boolean isLogBase2()
	{
		return true;
	}
	
	public int getShuffleAcrossAllDataNum()
	{
		return shuffleAcrossAllDataNum;
	}
	
	public File getCelFileDirectory() throws Exception
	{
		return null;
	}
	
	public void shuffleAcrossAllData() throws Exception
	{
		getRMAResults();
		
		if( randomRmaGep != null )
			randomRmaGep.scrambleData();
		
		if( qqRmaGep != null )
			qqRmaGep.scrambleData();
		
		if( nonQQRmaGep != null )
			nonQQRmaGep.scrambleData();
		
		shuffleAcrossAllDataNum++;
	}
	
	public boolean isUseQuantileQuantileNormalization()
	{
		return useQuantileQuantileNormalization;
	}
	
	public void setUseQuantileQuantileNormalization(
			boolean useQuantileQuantileNormalization)
	{
		this.useQuantileQuantileNormalization = useQuantileQuantileNormalization;
	}
	
	public boolean isUseRandomRmaData()
	{
		return useRandomRmaData;
	}
	
	public void setUseRandomRmaData(boolean useRandomRmaData)
	{
		this.useRandomRmaData = useRandomRmaData;
	}
	
	public File getGinFile() throws Exception
	{
		return null;
	}
	
	public boolean isUseStatisticLevelNormalization()
	{
		return useStatisticLevelNormalization;
	}
	
	public void setUseStatisticLevelNormalization(
			boolean useStatisticLevelNormalization)
	{
		this.useStatisticLevelNormalization = useStatisticLevelNormalization;
	}
	
	abstract public String getShortExperimentName() throws Exception;
	
	/*
	 * Subclasses must override if the experimentDescription list is to be mutable
	 */
	public void setExperimentFiles(List<String> list) throws Exception
	{
		throw new Exception(this.getClass().getName() 
				+ " does not support a mutable experimentDescription list");
	}
	
	/*
	 * Subclasses must override if the baselineDescription list is to be mutable
	 */
	public void setBaselineFiles(List<String> list) throws Exception
	{
		throw new Exception(this.getClass().getName() 
				+ " does not support a mutable baselineDescription list");
	}
	
	
	public String getExperimentDescripton() throws Exception
    {
		StringBuffer buff = new StringBuffer( getShortExperimentName() + "_");
		
		if( useRandomRmaData )
			buff.append("randomRMA_");
		else
			buff.append("RMA_");
		
		if( !useRandomRmaData && useQuantileQuantileNormalization )
			buff.append("QQNorm_");
		else
			buff.append("NoQQNorm_");
		
		if( !useRandomRmaData && useStatisticLevelNormalization )
			buff.append( "StatNorm_" );
		else
			buff.append("NoStatNorm_");
		
		if( shuffleAcrossAllDataNum > 0  )
			buff.append("shuffle" + shuffleAcrossAllDataNum +"_");
		
		buff.append(this.dataScope.getDescription());
		
		return buff.toString();
    }
    
	/*
	 *   Not thread safe
	 */
    public GeneExperimentParser getRMAResults() throws Exception
    {
    		if( useRandomRmaData )
    		{
    			if( randomRmaGep != null )
    				return randomRmaGep;
    			
    			randomRmaGep = new GeneExperimentParser( getRandomRmaGepFilePath() );
    			return randomRmaGep;
    		}
    		
    		if( useQuantileQuantileNormalization )
    		{
    			if( qqRmaGep != null )
    				return qqRmaGep;
    			
    			qqRmaGep = new GeneExperimentParser( getQQRmaFilePath() );
    			return qqRmaGep;
    		}
    		
    		if( nonQQRmaGep != null )
    			return nonQQRmaGep;
    		
    		nonQQRmaGep = new GeneExperimentParser( getNonQQRmaGepFilePath() );
    		return nonQQRmaGep;
    	}
    
    public void clearRMACache() throws Exception
    {
    		this.nonQQRmaGep = null;
    		this.qqRmaGep = null;
    		this.randomRmaGep = null;
    		shuffleAcrossAllDataNum=0;
    }
    
    abstract public List<String> getBaselineFiles() throws Exception;
    abstract public List<String> getExperimentFiles() throws Exception;
	
    public void setDataScope(DataScope dataScope) throws Exception
    {
    		this.dataScope = dataScope;
    }
    
    public DataScope getDataScope()
    {
    		return dataScope;
    }
    
    public File getCdfFile() throws Exception
    {
    	return null;
    }
    
    public String getChipId() throws Exception
    {
    	return null;
    }
    
    /**
     *   Can be null
     */
    abstract public AbstractSpikedControls getSpikeDetails();
}
