package launchers;

import java.io.File;
import java.util.List;

import dataSetDescribers.PincProbe;
import dataSetDescribers.RatioExperiment;

import scripts.RunExperiments;
import utils.ConfigReader;

public class PincLauncher
{

	static void checkCommonConfigFileSettings() throws Exception
	{
		File pincDir = new File( ConfigReader.getPINCDirectory());
		
		if( ! pincDir.exists())
			throw new Exception("Could not find " + pincDir.getAbsolutePath());
		
		if( ! pincDir.isDirectory() )
			throw new Exception("Expecting a directory for " + pincDir.getAbsolutePath());
		
		File expectedFile = new File( pincDir.getAbsolutePath() + File.separator +  
				"rScripts" + File.separator + "GCRMA-CYBERT-FDR BM.R" );
		
		if( ! expectedFile.exists())
			throw new Exception("Expecting to find " + expectedFile.getAbsolutePath() +
					"is PINC_DIR set correctly in properties file?");
		
		File outputDir = new File(ConfigReader.getAffyAnalysisOutputDir());
		
		if( ! outputDir.exists())
			throw new Exception("Error! AFFY_ANALYSIS_OUTPUT_DIR " + outputDir.getAbsolutePath() +
					" does not exist!");
	
	}
	
	/**
	 *This will launch Pinc.
	 *
	 *Hopefully you have your AffyAnalysis.properties file set up properly.
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception
	{
		checkCommonConfigFileSettings();
		
		List<? extends RatioExperiment> reList = PincProbe.getAllPossibleComparions();
		
		for( RatioExperiment re : reList )
		{
			System.out.println("Starting " + re.getExperimentDescripton());
			re.setUseRandomRmaData(false);
			RunExperiments.runCyberTPaired(re);
			System.out.println("Finsihed " + re.getExperimentDescripton());
		}
	}
	

}
