package launchers;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import parsers.GeneExperimentParser;
import parsers.ScoreFileLineParser;

import scripts.WriteScores;
import statistics.CyberT;
import statistics.StatisticsLevelNormalization;
import statistics.StatisticsProvider;
import utils.ConfigReader;

import dataSetDescribers.AbstractSpikedControls;
import dataSetDescribers.FilePathGenerator;
import dataSetDescribers.HGU133ALatinSquare;
import dataSetDescribers.RatioExperiment;

public class Scheme4Launcher extends RatioExperiment
{

	private List<String> parseCommaSeparatedProperty(String inString)
	{
		List<String> list = new ArrayList<String>();
		
		StringTokenizer sToken = new StringTokenizer(inString, ",");
		
		while(sToken.hasMoreTokens())
			list.add(sToken.nextToken());
		
		return list;
	}
	
	@Override
	public List<String> getBaselineFiles() throws Exception
	{
		return parseCommaSeparatedProperty(ConfigReader.getBaselineChipNames());	
	}

	@Override
	public List<String> getExperimentFiles() throws Exception
	{
		return parseCommaSeparatedProperty(ConfigReader.getExperimentChipNames());
	}

	@Override
	public File getNonQQRmaGepFilePath() throws Exception
	{
		throw new Exception("Method not supported");
	}

	@Override
	public File getQQRmaFilePath() throws Exception
	{
		return new File(ConfigReader.getPathToLog2Data());
	}

	@Override
	public File getRandomRmaGepFilePath() throws Exception
	{
		throw new Exception("Method not supported");
	}

	@Override
	public String getShortExperimentName() throws Exception
	{
		File file = new File(ConfigReader.getPathToLog2Data());
		String name = file.getName();
		
		if( name.indexOf(".")!=-1)
			name = name.substring(0, name.lastIndexOf("."));
		
		return name;
	}

	@Override
	public AbstractSpikedControls getSpikeDetails()
	{
		return new HGU133ALatinSquare();
	}
	
	@Override
	public String getExperimentDescripton() throws Exception
    {
		String name = getShortExperimentName() ;
		
		if( isUseStatisticLevelNormalization())
			name = name + "_Scheme4";
		
		return name;
    }
	
	private static void checkConfigFile(RatioExperiment re) throws Exception
	{	
		if( re.getBaselineFiles().size() < 3)
			throw new Exception("Must have at least three comma-separated baseline files set in "
					+  ConfigReader.BASELINE_CHIP_NAMES );
		
		if( re.getExperimentFiles().size() < 3)
			throw new Exception("Must have at least three comma-separated experiment files set in " +
					ConfigReader.EXPERIMENT_CHIP_NAMES);
		
		GeneExperimentParser gep = re.getRMAResults();
		
		for( String s : re.getBaselineFiles() )
			if( ! gep.getExperimentNames().contains(s) )
				throw new Exception("Could not find baseline name " + s +".  Error in " + 
										ConfigReader.BASELINE_CHIP_NAMES + " in the property file");
		
		for( String s : re.getExperimentFiles() )
			if( ! gep.getExperimentNames().contains(s) )
				throw new Exception("Could not find experiment name " + s +".  Error in " + 
										ConfigReader.EXPERIMENT_CHIP_NAMES+ " in the property file");
		
		
	}
	
	public static void main(String[] args) throws Exception
	{
		PincLauncher.checkCommonConfigFileSettings();
		
		RatioExperiment re = new Scheme4Launcher();
		
		// this assumes that the QQ normalization has already been done
		// (used for naming conventions in previous versions of the code
		// will not have any effect if the user has in fact not done QQ on 
		// their dataset
		re.setUseQuantileQuantileNormalization(true);
		
		re.setUseRandomRmaData(false);
		re.setUseStatisticLevelNormalization(false);
		
		checkConfigFile(re);
		StatisticsProvider sp = new CyberT();
		WriteScores.writeAScore(re, sp);
		
		ScoreFileLineParser.convertFromBinaryFile(re,sp,false, false);
		
		re.setUseStatisticLevelNormalization(true);
		new StatisticsLevelNormalization(re,sp);
		ScoreFileLineParser.convertFromBinaryFile(re,sp,false, false);
		
		// delete the .jitn files so they don't clutter up the output
		FilePathGenerator.getScoreFile(re,sp).delete();
		re.setUseStatisticLevelNormalization(false);
		FilePathGenerator.getScoreFile(re,sp).delete();
	}
}
