/*
 * Copyright @ 2001-2003, The Institute for Genomic Research (TIGR).
 * All rights reserved.
 *
 *
 * Lowess.java
 *
 * Created on December 10, 2001, 10:39 AM
 * @author  wliang
 * @version
 */

package org.tigr.midas.engine;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.StringTokenizer;

import Jama.Matrix;

public class Lowess {
    
    private static float[] xArray;
    private static float[] yArray;
    private static float[] yEst;
    private float[] xSub;
    private float[] ySub;
    private int subQueryPointIndex;
    private float[] weights;
    
    public Lowess(float[] xInput, float[] yInput, int windowSize) {
        
        xArray = xInput;
        yArray = yInput;
        int totalDataCount = xArray.length;
        yEst = new float[totalDataCount];
        
        if (windowSize > 1){    //Issue Locfit2.0, 08-01-2002
            xSub = new float[windowSize];
            ySub = new float[windowSize];
            weights = new float[windowSize];
            
            System.out.println("    ---- Bandwidth = " + windowSize);
            System.out.println("    ---- Please be patient, I am working hard, you know it's not an easy computation ...");
            
            //Construct matrix
            Matrix WX = new Matrix(windowSize, 2);
            Matrix WY = new Matrix(windowSize, 1);
            
            for (int q = 0; q < totalDataCount; q++){
                
                getEstWindowSubArrayAndWeights(q, windowSize);
                
                //Construct matrix
                //Matrix X = new Matrix(windowSize, 2);
                //Matrix WX = new Matrix(windowSize, 2);
                //Matrix WY = new Matrix(windowSize, 1);
                
                for ( int row = 0; row < windowSize; row++ ) {
                    WX.set(row, 0, Math.sqrt(weights[row]));
                    WX.set(row, 1, Math.sqrt(weights[row]) * xSub[row]);
                    WY.set(row, 0, Math.sqrt(weights[row]) * ySub[row]);
                }
                
                //Apply first order estimation: local linear model
                Matrix WXT = WX.transpose();
                Matrix B = (WXT.times(WX)).inverse().times((WXT).times(WY));
                
                float B0 = (float)B.get(0, 0);
                float B1 = (float)B.get(1, 0);
                yEst[q] = B0 + B1 * xSub[subQueryPointIndex];
            }
        }else{ //windowSize = 0 or 1 //Issue Locfit2.0, 08-01-2002
            for (int q = 0; q < totalDataCount; q++){
                yEst[q] = 0;
            }
        }
    }
    
    public float[] getXarray(){
        return xArray;
    }
    
    public float[] getYEst(){
        return yEst;
    }
    
    private void getEstWindowSubArrayAndWeights(int queryPointIndex, int window){
        
        int fullLength = xArray.length;
        int subStartIndex = 0;
        int windowSize = window;
        float maxDistance = 0;
        
        //Issue Locfit1.0, 08-01-2002
        //if ((queryPointIndex >= windowSize) && (queryPointIndex <= fullLength - windowSize)){//middle region data
        if ((queryPointIndex >= windowSize) && (queryPointIndex < fullLength - windowSize)){//middle region data
            for (int i = 0; i < windowSize; i++){
                if ( (xArray[queryPointIndex] - (xArray[queryPointIndex - windowSize + 1 + i])) <= (xArray[queryPointIndex + 1 + i] - xArray[queryPointIndex]) ){
                    subStartIndex = queryPointIndex - windowSize + 1 + i;
                    subQueryPointIndex = queryPointIndex - subStartIndex;
                    maxDistance = ( ( (xArray[queryPointIndex] - xArray[subStartIndex]) > (xArray[subStartIndex + windowSize - 1]) - xArray[queryPointIndex] ) ? (xArray[queryPointIndex] - xArray[subStartIndex]) : (xArray[subStartIndex + windowSize - 1] - xArray[queryPointIndex] ) );
                    break;
                }
            }
        }else if (queryPointIndex < windowSize){
            for (int i = 0; i < windowSize; i++){
                if ( (xArray[queryPointIndex] - xArray[i]) <= (xArray[windowSize + i] - xArray[queryPointIndex]) ){
                    subStartIndex = i;
                    subQueryPointIndex = queryPointIndex - subStartIndex;
                    maxDistance = ( (xArray[queryPointIndex] - xArray[subStartIndex]) > (xArray[subStartIndex + windowSize - 1] - xArray[queryPointIndex] ) ) ? (xArray[queryPointIndex] - xArray[subStartIndex]) : (xArray[subStartIndex + windowSize - 1] - xArray[queryPointIndex] );
                    break;
                }
            }
        }else{//right margin data
            for (int i = 0; i < windowSize; i++){
                if ( (xArray[fullLength - 1 - i] - xArray[queryPointIndex]) <= (xArray[queryPointIndex] - xArray[fullLength - windowSize - 1 - i]) ){
                    subStartIndex = fullLength - windowSize - i;
                    subQueryPointIndex = queryPointIndex - subStartIndex;
                    maxDistance = ( (xArray[queryPointIndex] - xArray[subStartIndex]) > (xArray[subStartIndex + windowSize - 1] - xArray[queryPointIndex] ) ) ? (xArray[queryPointIndex] - xArray[subStartIndex]) : (xArray[subStartIndex + windowSize - 1] - xArray[queryPointIndex] );
                    break;
                }
            }
        }
        
        System.arraycopy(xArray, subStartIndex, xSub, 0, windowSize);
        System.arraycopy(yArray, subStartIndex, ySub, 0, windowSize);
        
        //Calculate distances and find tricube weights
        for (int j = 0; j < windowSize; j ++) {
            float distance = Math.abs(xSub[subQueryPointIndex] - xSub[j]);
            weights[j] = (float)Math.pow(1.0 - Math.pow((distance / maxDistance), 3.0), 3.0);
        }
    }
    
    /*
    @SuppressWarnings("unchecked")
	public static void main(String[] args) throws Exception
    {
    	BufferedWriter writer = new BufferedWriter(new FileWriter("c:\\temp\\checkLoess.txt"));
    	writer.write("sum\tscore\testimate\n");
    	RatioExperiment rei = new LatinSquareGeneric(8,9);
		rei.setDataScope(DataScope.NULLS_ONLY);
		StatisticsProvider sp = new ShrunkenVariance(1);
		rei.setUseQuantileQuantileNormalization(false);
		rei.setUseStatisticLevelNormalization(false);
		List<ScoreFileLineParser> resultsList = ScoreFileLineParser.parseFile(rei,sp);
		
		Collections.sort( resultsList, new ScoreFileLineParser.SortByAverage() );
		
		float[] xInput = new float[resultsList.size()];
		float[] yInput = new float[resultsList.size()];
		
		for( int x=0; x < resultsList.size(); x++ )
		{
			ScoreFileLineParser sflp = resultsList.get(x);
			xInput[x] = sflp.getAverage();
			yInput[x] = sflp.getScore();
		}
		
		Lowess lowess = new Lowess(xInput,yInput,750);
		
		float[] yEst = lowess.getYEst();
		
		for( int x=0; x < xInput.length; x++)
		{
			writer.write( xInput[x] + "\t" + yInput[x] + "\t" + yEst[x] + "\n" );
		}
		
		writer.flush();  writer.close();
    }
    */
    
    public static void main(String[] args) throws Exception
    {
    	BufferedReader reader = new BufferedReader(new FileReader(new File( "c:\\temp\\quickOut.txt" )));
    	float[] x = new float[22182];
    	float[] y = new float[22182];
    	
    	for( int i=0; i < 22182; i++)
    	{
    		String nextLine = reader.readLine();
    		StringTokenizer sToken = new StringTokenizer(nextLine);
    		x[i] = Float.parseFloat(sToken.nextToken());
    		y[i] = Float.parseFloat(sToken.nextToken());
    	}
    	
    	new Lowess(x,y,1000);
    }
}
