
package parsers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;

import utils.ConfigReader;


import dataSetDescribers.RatioExperiment;


public class CelParserFileLine
{
    
    public static final float EMPTY = -99.999f;
    
    private final int x;
    private final int y;
    private final float mean;
    private final float stdev;
    
    public CelParserFileLine(String inLine) throws Exception
    {
        StringTokenizer sToken = new StringTokenizer(inLine);
        this.x = Integer.parseInt(sToken.nextToken());
        this.y = Integer.parseInt(sToken.nextToken());
        this.mean = Float.parseFloat(sToken.nextToken());
        
        if ( sToken.hasMoreTokens())
            this.stdev = Float.parseFloat(sToken.nextToken());
        else
            this.stdev = 0;
        
       // if ( sToken.hasMoreTokens())
         //   throw new Exception("Unexpected token " + sToken.nextToken());
    }
    
    /**
     *   The key is x + "_" + y.
     *   The value is a CelParserFileLine line
     */
    public static HashMap<String, CelParserFileLine> parseFileForMap(File file) throws Exception
    {
        HashMap<String, CelParserFileLine> map = new HashMap<String, CelParserFileLine>();
            
        BufferedReader reader = new BufferedReader(new FileReader(file));
            
        String nextLine = reader.readLine();
            
        while( ! nextLine.startsWith("[INTENSITY]"))
            nextLine = reader.readLine();
            
        reader.readLine();  reader.readLine();
        nextLine = reader.readLine();
            
        while ( nextLine.trim().length() > 0 )
        {
            CelParserFileLine cpfl = new CelParserFileLine(nextLine);
            String key = cpfl.getX() + "_" + cpfl.getY();
            
            if ( map.containsKey(key))
                throw new Exception("Found two for " + key);
            
            map.put(key, cpfl);
            nextLine = reader.readLine();
        }            
        
        return map;
    }
    
    /*
     *   Not thread safe
     */
    public static List<float[][]> parseFilesForArrays(List<File> files) throws Exception
    {
        List<float[][]> list= new ArrayList<float[][]>();
        
        for ( int x=0; x < files.size(); x++ )
        {
            File file = (File) files.get(x);
            System.out.println("Parsing " + file.getAbsolutePath() );
            list.add( parseFileForArray(file ));
        }
        
        return list;
    }
    	
    public static float[][] parseFilesForAverage(List<File> files) throws Exception
    {

        float[][] returnArray = new float[ConfigReader.getXDim()][ConfigReader.getYDim()];
        
        for ( int x=0; x < returnArray.length; x++ )
            for ( int y=0; y < returnArray[x].length; y++ )
                returnArray[x][y] = 0;
        
        List<float[][]> listToAverage = parseFilesForArrays(files);
        
        for( float[][] subArray : listToAverage )
        {
        	for( int x=0; x < returnArray.length; x++)
        		for( int y=0; y < returnArray[x].length; y++)
        			returnArray[x][y] += subArray[x][y];
        }
        
    	for( int x=0; x < returnArray.length; x++)
    		for( int y=0; y < returnArray[x].length; y++)
    			returnArray[x][y] /= listToAverage.size();
        
        return returnArray;
    }
    
    
    	
    public static List<float[][]> getExperimentFiles( RatioExperiment re )
    	throws Exception
    {
    	List<File> fileList = new ArrayList<File>();
    	
    	for( String file : re.getExperimentFiles() )
    		fileList.add( new File(re.getCelFileDirectory() + File.separator + 
    				file) );
    	
    	return parseFilesForArrays(fileList) ;
    }
    
    public static List<float[][]> getBaselineFiles( RatioExperiment re )
		throws Exception
	{
		List<File> fileList = new ArrayList<File>();
		
		for( String file : re.getBaselineFiles() )
			fileList.add( new File(re.getCelFileDirectory() + File.separator + 
					file) );
		
		return parseFilesForArrays(fileList) ;
	}
    	
    private static HashMap<File, float[][]> celFiles = new 
    			HashMap<File, float[][]>();
    	
    /**
     *  Not thread safe.  Keeps an internal cache of previously read cel files.
     * 
     */
    public static float[][] parseFileForArray(File file) throws Exception 
    {
    	if ( celFiles.containsKey(file) )
    		return celFiles.get(file);
    	
        float[][] returnArray = new float[ConfigReader.getXDim()][ConfigReader.getYDim()];
        
        for ( int x=0; x < returnArray.length; x++ )
            for ( int y=0; y < returnArray.length; y++ )
                returnArray[x][y] = EMPTY;
        
        BufferedReader reader = new BufferedReader(new FileReader(file));
        
        String nextLine = reader.readLine();
        
        while( ! nextLine.startsWith("[INTENSITY]"))
            nextLine = reader.readLine();
        
        reader.readLine();  reader.readLine();
        nextLine = reader.readLine();
        
        while ( nextLine.trim().length() > 0 )
        {
            CelParserFileLine cpfl = new CelParserFileLine(nextLine);
            returnArray[cpfl.getX()][cpfl.getY()] = cpfl.getMean();
            nextLine = reader.readLine();
        }
        
        celFiles.put(file, returnArray);
        
        return returnArray;
    }
    
    public float getCv()
    {
        return stdev / mean;
    }
    
    public float getMean()
    {
        return mean;
    }
    
    public int getX()
    {
        return x;
    }
    
    public int getY()
    {
        return y;
    }
    
    public float getStdev()
    {
        return stdev;
    }
    
    public static float getMean(List<CelParserFileLine> list)
    {
        float sum =0;
        int n = 0;
        
        for ( int x=0; x < list.size(); x++)
        {
            CelParserFileLine cpfl = (CelParserFileLine) list.get(x);
            sum += cpfl.getMean();
            n++;
        }
        
        return sum / n;
    }
    
    public static float getMean(float[][] vals)
    {
        float sum =0;
        int n = 0;
        
        for ( int x=0; x < vals.length; x++)
            for ( int y=0; y < vals[x].length; y++)
        {
            sum += vals[x][y];
            n++;
        }
        
        return sum / n;
    }
    
    public static void main(String[] args) throws Exception
	{
		List<File> fileList = new ArrayList<File>();
		fileList.add(new File("/Users/afodor/Julie/GenomeBiologyCelFiles/qqnorm/G4 B.cel"));
		float[][] f = parseFilesForAverage(fileList);
		
		for (int x=0; x < f.length; x++)
			for( int y=0; y < f[x].length; y++)
				System.out.println( x + " " + y +  " " +  f[x][y] );
	}
    
    /*
    public static void main(String[] args) throws Exception
    {
        File aDir = new File("C:\\AldrichMicroArray\\exports");
        String[] names = aDir.list();
        
        for ( int x=0; x < names.length; x++ )
        {
            String subName = names[x].substring(names[x].indexOf("_") + 1, names[x].lastIndexOf("."));
            BufferedWriter writer = new BufferedWriter(new FileWriter(new File("c:\\temp\\" + subName +".txt")));
            writer.write( subName + "_x\t");
            writer.write( subName + "_y\t");
            writer.write( subName + "value\n");
            
            File aFile = new File(aDir.getAbsolutePath() + File.separator + names[x]);
            float[][] vals= parseFileForArray(aFile);
            System.out.println( names[x] + " " + getMean(vals) );
            
            for ( int y=0; y < vals.length; y++ )
                for ( int z=y+1; z< vals[y].length; z++ )
                {
                    writer.write(y + "\t");
                    writer.write(z + "\t");
                    writer.write(vals[y][z] + "\n");
                }
            
            writer.flush();  writer.close();
        }
    }
    */
}
