package parsers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class RocFileLine
{
	private final float score;
	private final float truePositiveRate;
	private final float falsePositiveRate;
	
	private RocFileLine(String inString) throws Exception
	{
		StringTokenizer sToken = new StringTokenizer(inString);
		
		this.score = Float.parseFloat(sToken.nextToken());
		this.truePositiveRate = Float.parseFloat(sToken.nextToken());
		this.falsePositiveRate = Float.parseFloat(sToken.nextToken());
		
		if( sToken.hasMoreTokens())
			throw new Exception("Parsing error");
	}
	
	public static float getBestTruePositiveRate(File file, float maxAllowedFalseNegativeRate) throws Exception
	{
		 List<RocFileLine> list= parseFile(file);
		 
		 float returnVal = 0;
		 
		 for( RocFileLine fileLine : list )
			 if( fileLine.getFalsePositiveRate() <= maxAllowedFalseNegativeRate )
				 returnVal = Math.max ( returnVal, fileLine.getTruePositiveRate() );
		 
		 return returnVal;
	}
	
	public static List<RocFileLine> parseFile(File file) throws Exception
	{
		List<RocFileLine> returnList = new ArrayList<RocFileLine>();
		
		BufferedReader reader = new BufferedReader(new FileReader(file));
		
		reader.readLine();
		
		String nextLine = reader.readLine();
		
		while( nextLine != null)
		{
			returnList.add( new RocFileLine(nextLine) );
			nextLine = reader.readLine();
		}
		
		reader.close();
		
		return returnList;
	}

	public float getFalsePositiveRate()
	{
		return falsePositiveRate;
	}

	public float getScore()
	{
		return score;
	}

	public float getTruePositiveRate()
	{
		return truePositiveRate;
	}
	
	

}
