package scripts;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import parsers.CdfFileLine;
import parsers.CelParserFileLine;
import utils.ConfigReader;
import utils.TTest;
import dataSetDescribers.LatinSquareGeneric;
import dataSetDescribers.RatioExperiment;

public class ProbeMeanVsSD 
{
	private static final double LOG_2 = Math.log(2.0);
	
   public static double log2(double d) 
   {
      return Math.log(d)/ LOG_2 ;
   }
   
	public static void main(String[] args) throws Exception
	{
		RatioExperiment re = new LatinSquareGeneric(1,2);
		
		BufferedWriter writer = new BufferedWriter(new FileWriter(new File( 
			ConfigReader.getAffyAnalysisOutputDir() + File.separator + 
			"Probe1vs2_LoggedMeanVsSd")));
		
		writer.write("x\ty\ttScore\tbaselineAverage\texperimentAverage\toverallAverage\tdifference\tpooledVar\n");
		
		HashMap<String, List<CdfFileLine>> cdfMap = 
			CdfFileLine.parseFileForAll( re.getCdfFile(), re.getChipId());
		
		List<float[][]> experimentCels = CelParserFileLine.getExperimentFiles(re);
		List<float[][]> baselineCels = CelParserFileLine.getBaselineFiles(re);
		
		int index =0;
		int n = experimentCels.size() + baselineCels.size();
		
		for( String key : cdfMap.keySet())
			if( ! re.getSpikeDetails().getAllSpikedGeneIds().contains(key) )
				if( ! re.getSpikeDetails().isExcludedProbeSet(key))
					for(  CdfFileLine fileLine : cdfMap.get(key))
					{
						List<Float> expVals = new ArrayList<Float>();
						List<Float> baseVals = new ArrayList<Float>();
						
						for( float[][] f : experimentCels )
							expVals.add((float) log2( f[fileLine.getX()][fileLine.getY()]));
						
						for( float[][] f : baselineCels)
							baseVals.add( (float) log2( f[fileLine.getX()][fileLine.getY()]));
						
						double average = 
							( TTest.getSum(expVals) + TTest.getSum(baseVals) )/n;

						double difference = 
							TTest.getAverage(expVals) -TTest.getAverage(baseVals);
						
						double pooledSd = TTest.getPooledVariance(expVals, baseVals);
							
						writer.write(fileLine.getX() + "\t" + fileLine.getY() + "\t");
						writer.write(TTest.ttest(baseVals, expVals).getScore() + "\t");
						writer.write(TTest.getAverage(baseVals) + "\t");
						writer.write(TTest.getAverage(expVals) + "\t");
						writer.write( average + "\t");
						writer.write( difference + "\t");
						writer.write(  pooledSd  + "\n");
						
						
						if( ++index % 10000 == 0 ) 
							System.out.println(index );
					}
		
		writer.flush();  writer.close();
	}
}
