package scripts;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.List;

import parsers.NumTrueAndFalsePositives;
import parsers.ScoreFileLineParser;

import statistics.CyberT;
import statistics.CyberTPairedTakeUnloggedDifference;
import statistics.ShrunkenVariance;
import statistics.StandardT;
import statistics.StatisticsLevelNormalization;
import statistics.StatisticsProvider;
import utils.ConfigReader;

import dataSetDescribers.DataScope;
import dataSetDescribers.FilePathGenerator;
import dataSetDescribers.RatioExperiment;

public class RunExperiments
{
	public static void main(String[] args) throws Exception
	{
		
	}
	
	public static void dumpCounts(List<RatioExperiment> inList, String filePrefix) 
		throws Exception
	{
		BufferedWriter writer = new BufferedWriter(new FileWriter(new File(
			ConfigReader.getAffyAnalysisOutputDir() + File.separator + 
		  filePrefix + 	"_Counts.txt")));
		
		writer.write("experiment\tcyberTQQCount\tcyberTNormCount\tstandardT\n");
		
		for( RatioExperiment re : inList )
		{
			re.setDataScope(DataScope.ALL);
			re.setUseQuantileQuantileNormalization(true);
			re.setUseStatisticLevelNormalization(false);
			
			writer.write( re.getShortExperimentName() +"\t" );
			
			StatisticsProvider sp = new CyberT();
			
			NumTrueAndFalsePositives nfp = 
				ScoreFileLineParser.getPositiveCount(re, sp, 0.1f, false);
			
			writer.write(nfp.getNumChosen() + "\t");
			
			re.setUseStatisticLevelNormalization(true);
			
			nfp = 
				ScoreFileLineParser.getPositiveCount(re, sp, 0.1f, false);
			
			writer.write(nfp.getNumChosen() + "\t");
			
			re.setUseQuantileQuantileNormalization(true);
			re.setUseStatisticLevelNormalization(false);
			sp = new StandardT();
			
			nfp = 
				ScoreFileLineParser.getPositiveCount(re, sp, 0.1f, false);
			
			writer.write(nfp.getNumChosen() + "\n");
			
			writer.flush();
			System.out.println("Finished " + re.getShortExperimentName());
		}
		
		writer.flush();  writer.close();
	}
	
	public static void runStandardT(RatioExperiment re) throws Exception
	{
		re.setUseQuantileQuantileNormalization(true);
		re.setUseStatisticLevelNormalization(false);
		re.setDataScope(DataScope.ALL);
		
		StatisticsProvider sp = new StandardT();
		WriteScores.writeAScore(re, sp);
		ScoreFileLineParser.convertFromBinaryFile(re, sp,false,false);
		System.out.println( re.getExperimentDescripton() );
		
	}
	
	public static void runAllShrunkenTs( RatioExperiment re ) throws Exception
	{
		float[] bs = {0f,.2f,.4f,.6f,.8f, 1.0f};
		
		re.setDataScope(DataScope.ALL);
		re.setUseRandomRmaData(false);
		re.setUseQuantileQuantileNormalization(true);
		re.setUseStatisticLevelNormalization(true);
		
		for( int i=0; i < bs.length; i++)
		{
			StatisticsProvider sp = new ShrunkenVariance(bs[i]);
			WriteScores.writeAScore(re, sp);
		}
	}
	
	public static void runCyberT(RatioExperiment re) throws Exception
	{
		re.setUseQuantileQuantileNormalization(true);
		re.setUseStatisticLevelNormalization(false);
		re.setDataScope(DataScope.ALL);
		
		StatisticsProvider sp = new CyberT();
		WriteScores.writeAScore(re, sp);
		ScoreFileLineParser.convertFromBinaryFile(re,sp,false, false);
	}
	
	public static void runCyberTPaired(RatioExperiment re) throws Exception
	{
		if( ! re.isLogBase2())
			throw new Exception("Cyber t paired assume log base 2 data");
		
		re.setUseQuantileQuantileNormalization(true);
		re.setUseStatisticLevelNormalization(false);
		re.setDataScope(DataScope.ALL);
		
		StatisticsProvider sp = new CyberTPairedTakeUnloggedDifference();
		
		WriteScores.writeAScore(re, sp);
		ScoreFileLineParser.convertFromBinaryFile(re,sp,false, false);
		
		re.setUseStatisticLevelNormalization(true);
		new StatisticsLevelNormalization(re,sp);
		ScoreFileLineParser.convertFromBinaryFile(re,sp,false, false);
		
		// delete the .jitn files so they don't clutter up the output
		FilePathGenerator.getScoreFile(re,sp).delete();
		re.setUseStatisticLevelNormalization(false);
		FilePathGenerator.getScoreFile(re,sp).delete();
	}
}
