package scripts;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


import dataSetDescribers.ExperimentData;
import dataSetDescribers.FilePathGenerator;
import dataSetDescribers.ProbeSetData;
import dataSetDescribers.RatioExperiment;

import statistics.StatisticsLevelNormalization;
import statistics.StatisticsProvider;
import utils.StatisticReturnObject;
import utils.TTest;

public class WriteScores
{
	private static void setPValues(StatisticsProvider sp, ExperimentData experiment)
		throws Exception
	{
		if ( sp.hasPValue() )
		{
			for( ProbeSetData probeSet : experiment.getProbeSetData() )
			{
				StatisticReturnObject sro = sp.getStats(probeSet);
				probeSet.setCalculatedScore(sro.getScore());
				probeSet.setSortableStorage( sro.getPValue() );
			}
		}
		else
		{
			List<Double> scores = new ArrayList<Double>();
			
			for( ProbeSetData probeSet : experiment.getProbeSetData() )
			{
				StatisticReturnObject sro = sp.getStats(probeSet);
				probeSet.setCalculatedScore(sro.getScore());
				scores.add( probeSet.getCalculatedScore() );
			}
			
			double mu = TTest.getAverage(scores);
			double sd = TTest.getStDev(scores);
			
			for( ProbeSetData probeSet : experiment.getProbeSetData() )
			{
				double pValueFromNorm = StatisticsLevelNormalization.getPValueFromScore( 
						probeSet.getCalculatedScore(), mu,sd);
				
				probeSet.setSortableStorage( pValueFromNorm );
			}
		}
	}
	
	private static void writeResults(ExperimentData experiment, StatisticsProvider sp,
			RatioExperiment rei) throws Exception
	{
		if( rei.isUseStatisticLevelNormalization())
			throw new Exception("Use StatisticsLevelNormalization class instead");
		
		 DataOutputStream out = new DataOutputStream( new BufferedOutputStream(
         	new FileOutputStream( FilePathGenerator.getScoreFile(rei,sp))));
		
		// first integer is number of lines in the file
		 out.writeInt(experiment.getProbeSetData().size());
		 
		double byConstant = 0;
		
		for ( int x=0; x <  experiment.getProbeSetData().size() ; x++)
		{
			byConstant += (1.0/(x+1));
		}
		
		for( int x=0; x< experiment.getProbeSetData().size(); x++)
		{
			ProbeSetData probeSet = experiment.getProbeSetData().get(x);
			out.writeUTF( probeSet.getProbeSetId());
			
			if( rei.getSpikeDetails() == null || ! rei.getSpikeDetails().getAllSpikedGeneIds().contains( probeSet.getProbeSetId() ) )
				out.writeBoolean(false );
			else
				out.writeBoolean(true);
			
			out.writeFloat( probeSet.getSum() / 
					(probeSet.getExperimentData().size() + probeSet.getBaselineData().size() )  ); 
			out.writeDouble( probeSet.getCalculatedScore());
			
			double unnormalizedFoldChange =  0;
			
			if( rei.isLogBase2() )
				unnormalizedFoldChange = Math.pow(2, probeSet.getExperimentAverage() 
														-probeSet.getBaselineAverage());
			else
				unnormalizedFoldChange = Math.exp(probeSet.getExperimentAverage() -
													probeSet.getBaselineAverage());
			
			out.writeDouble( unnormalizedFoldChange );
			
			out.writeDouble( probeSet.getSortableStorage() );
			out.writeDouble( (x+1.0) / experiment.getProbeSetData().size() );
			out.writeFloat( probeSet.getExpectedScore() );
			out.writeFloat( 0f);
			double bhFdr = (experiment.getProbeSetData().size() * probeSet.getSortableStorage())/(x+1);
			out.writeDouble( bhFdr );
			out.writeDouble(byConstant*bhFdr);
		}
		
		out.flush();  out.close();
	}
	
	
	public static void writeAScore( RatioExperiment rei, StatisticsProvider sp ) throws Exception
	{
		
		ExperimentData experiment = ProbeSetData.getListFromRMA(rei);
		sp.preliminaryCalculations(experiment);
		
		setPValues(sp, experiment);
		Collections.sort( experiment.getProbeSetData() );
		writeResults(experiment,sp,rei);
	}
	
}
