package statistics;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import rScripts.ShellToR;

import utils.ConfigReader;
import utils.StatisticReturnObject;
import dataSetDescribers.ExperimentData;
import dataSetDescribers.FilePathGenerator;
import dataSetDescribers.ProbeSetData;

/*
 *   This works by shelling out to R
 */
public class CyberT implements StatisticsProvider
{
	private List<Double> tValues =new ArrayList<Double>();
	private List<Double> pValues = new ArrayList<Double>();
	private HashMap<String, Integer> positions = new HashMap<String, Integer>();
	private boolean prelimCalculationsDone = false;
	
	// for testing
	public List<Double> getPValues()
	{
		return Collections.unmodifiableList( pValues );
	}
	
	// for testing
	public List<Double> getTValues()
	{
		return Collections.unmodifiableList(tValues);
	}
	
	// for testing
	public Map<String,Integer> getPositions()
	{
		return Collections.unmodifiableMap(positions);
	}
	
	// for testing
	public HashMap<Integer, String> reversePositions() throws Exception
	{
		HashMap<Integer, String> map = new HashMap<Integer, String>();
		
		for( String key : positions.keySet() )
		{
			Integer anInt = positions.get(key);
			
			if( map.get(anInt) != null)
				throw new Exception("Dual keys " + key);
			
			map.put( anInt, key );
		}
		
		return map;
	}
	private void parseResultsFile( File resultsFile, ExperimentData experiment ) throws Exception
	{
		System.out.println( "Parsing results file from cyberT " + resultsFile.getAbsolutePath() );
		BufferedReader reader = new BufferedReader(new FileReader(resultsFile));
		
		reader.readLine();
		
		String nextLine = reader.readLine();
		
		while( nextLine != null && nextLine.trim().length() != 0)
		{
			StringTokenizer sToken = new StringTokenizer(nextLine, ",");
			String indexString = sToken.nextToken();
			indexString = indexString.substring(1);
			indexString = indexString.substring(0, indexString.length() -1);
			
			int index = Integer.parseInt(indexString)-1;
			
			if (tValues.size() != index )
				throw new Exception("Parsing error " + tValues.size() + " " + index);
			
			if ( pValues.size() != index)
				throw new Exception("Parsing error");
			
			positions.put( experiment.getProbeSetData().get(index).getProbeSetId(), index );
			
			
			//int numToSkip = experiment.getNumberOfBaseline() + 4;
			int numToSkip = 11 + experiment.getNumberOfBaseline() + experiment.getNumberOfExperiments();
			
			for ( int x=1; x <= numToSkip ; x++)
				sToken.nextToken();
			
			tValues.add( Double.parseDouble(sToken.nextToken()));
			
			sToken.nextToken();  sToken.nextToken();
			
			pValues.add( Double.parseDouble(sToken.nextToken()));
			
			nextLine = reader.readLine();
		}
		
		reader.close();
	}
	
	protected boolean doLowessNormalization()
	{
		return false;
	}
	
	private void writeCommandFile( ExperimentData experiment  ) throws Exception
	{
		File commandFile = FilePathGenerator.deleteAndEnsureDeleted(
				ConfigReader.getAffyAnalysisOutputDir() + File.separator + "rCommand.txt");
		
		BufferedWriter writer = new BufferedWriter(new FileWriter(commandFile));
		
		File cyberTFunctions = new File(ConfigReader.getPINCDirectory() + File.separator + 
				"rScripts"  + File.separator + "GCRMA-CYBERT-FDR BM.R");
		
		writer.write( "source(\"" +  FilePathGenerator.getRPath(cyberTFunctions)  + "\")\n" );
		
		File headerlessData = FilePathGenerator.deleteAndEnsureDeleted( ConfigReader.getAffyAnalysisOutputDir()
				 + File.separator + "DataForROkToDelete.txt" );
		
		experiment.writeHeaderlessDataToFile(headerlessData, doLowessNormalization());
		
		writer.write( "datamatrix<-read.table(\"" + FilePathGenerator.getRPath( headerlessData ) + 
				"\")\n");
		
		writer.write( "NumberOfTests<-" + experiment.getNumberOfBaseline() + "\n" );
		writer.write( "NumberOfSecondTests<-" + experiment.getNumberOfExperiments() + "\n");
		
		writer.write(
		"bayesianResultMatrix2 <- bayesT(cbind(datamatrix[,1:(NumberOfTests)]" + 
		",datamatrix[,(1+NumberOfTests):(NumberOfTests+NumberOfSecondTests)])," + 
		"NumberOfTests,NumberOfSecondTests);\n");
		
		File rResults = FilePathGenerator.deleteAndEnsureDeleted( ConfigReader.getAffyAnalysisOutputDir()
				+ File.separator + "rResultsOkToDelete.txt");
		
		
		writer.write("write.csv(bayesianResultMatrix2, file=\"" + FilePathGenerator.getRPath(rResults)
				+ "\");\n" );
		
		writer.flush();  writer.close();
		ShellToR.shellToR(commandFile);
		parseResultsFile(rResults, experiment);
	}
	
	public void preliminaryCalculations(ExperimentData experiment) throws Exception
	{
		if( prelimCalculationsDone )
			throw new Exception("Preliminary Calculations should only be performed once");
		
		writeCommandFile(experiment);
		
		prelimCalculationsDone = true;
	}

	public StatisticReturnObject getStats(ProbeSetData probeSet) throws Exception
	{
		int index = positions.get(probeSet.getProbeSetId());
		
		return new StatisticReturnObject( tValues.get(index), pValues.get(index));
	}

	public boolean hasPValue()
	{
		return true;
	}

	public String getDescription()
	{
		return "CyberT";
	}

}
