package statistics;

import dataSetDescribers.ExperimentData;
import dataSetDescribers.ProbeSetData;
import utils.StatisticReturnObject;

public class ShrunkenVariance implements StatisticsProvider
{
	private final float b;
	private double overallVariance;
	private boolean calledPrelim = false;
	
	/* t statistics will be reported as:
	 * 
	 *         diff 
	 *  ----------------------------------
	 *   sqrt( B*VAR + ( 1 - B) * LocalVAR ) 
	 *   
	 *   where B is the shrinkage constant
	 *   VAR is the variance across the entire array
	 *   LocalVAR is the variance of one particular gene
	 *   diff is the mean difference between conditons.
	 *   
	 *   See Box 2 in Microarray data analysis: from disarray to consolidation and consenus.  Allison et al, 
	 *   Nature Review, Genetics, Vol 7, Jan. 2006, 55-65.
	 */
	public ShrunkenVariance(float b) throws Exception
	{
		this.b = b;
	}

	public String getDescription()
	{
		int intB = (int) (100.1 * b);
		return "bIs" + intB;
	}

	public StatisticReturnObject getStats(ProbeSetData probeSet) throws Exception
	{
		if( ! calledPrelim)
			throw new Exception("Must call Premilinary calculation before calling getStats()");
		
		
		double score =  probeSet.getShrunkenT(overallVariance, b);
		
		return new StatisticReturnObject(score,1);
	}

	public boolean hasPValue()
	{
		return false;
	}

	public void preliminaryCalculations(ExperimentData experiment) throws Exception
	{
		if( calledPrelim)
			throw new Exception("Do not call preliminary calculation twice");
		
		this.overallVariance = experiment.getAverageVaraiance();
		calledPrelim = true;
	}	
}
