package utils;

import java.io.InputStream;

import java.util.Properties;

public class ConfigReader
{
	public static final String PROPERTIES_FILE="AffyAnalysis.properties";
	
	private static ConfigReader configReader = null;
	private static Properties props = new Properties();
	
	public static final String TRUE="TRUE";
	public static final String YES="YES";
	
	public static final String AFFY_ANALYSIS_OUTPUT_DIR="AFFY_ANALYSIS_OUTPUT_DIR";
	public static final String FDR_ANALYSIS_OUTPUT_DIR="FDR_ANALYSIS_OUTPUT_DIR";
	public static final String R_DIRECTORY = "R_DIRECTORY";
	public static final String VERBOSE_CONSOLE = "VERBOSE_CONSOLE";
	public static final String LATIN_SQUARE_DIRECTORY = "LATIN_SQUARE_DIRECTORY";
	public static final String BLAST_DIRECTORY = "BLAST_DIRECTORY";
	public static final String NAEF_DIRECTORY = "NAEF_DIRECTORY";
	public static final String PERMUTATION_2_OUT_DIR = "PERMUTATION_2_OUT_DIR";
	public static final String NORMALIZED_AFFY_CEL_DIR=
		"NORMALIZED_AFFY_CEL_DIR";
	public static final String ALDRICH_DIR = "ALDRICH_DIR";
	public static final String AMPRNA_DIR="AMPRNA_DIR";
	public static final String CHOE_DIR="CHOE_DIR";
	public static final String TIM_DIR="TIM_DIR";
	public static final String JULIE_CEL_DIR="JULIE_CEL_DIR";
	public static final String CHRISTINE_CEL="CHRISTINE_CEL";
	public static final String DREAU_DIRECTORY="DREAU_DIRECTORY";
	public static final String SEQUENCE_DIR = "SEQUENCE_DIR";
	public static final String U95_LATIN_SQUARE_DIR="U95_LATIN_SQUARE_DIR";
	public static final String PROSTATE_DIR="PROSTATE_DIR";
	public static final String CDF_DIR="CDF_DIR";
	public static final String CDF_FILE="CDF_FILE";
	public static final String BASELINE_CELS_DIRECTORY="BASELINE_CELS_DIRECTORY";
	public static final String EXPERIMENT_CELS_DIRECTORY="EXPERIMENT_CELS_DIRECTORY";
	public static final String PINC_DIRECTORY="PINC_DIRECTORY";
	public static final String EXPECTED_NUMBER_PROBES = "EXPECTED_NUMBER_PROBES";
	
	public static final String BASELINE_CHIP_NAMES = "BASELINE_CHIP_NAMES";
	public static final String EXPERIMENT_CHIP_NAMES = "EXPERIMENT_CHIP_NAMES";
	public static final String PATH_TO_LOG_2_DATA = "PATH_TO_LOG_2_DATA";
	
	public static final String X_DIM = "X_DIM";
	public static final String Y_DIM = "Y_DIM";
	
	private static String getAProperty(String namedProperty ) throws Exception
	{
		Object obj = props.get( namedProperty );
		
		if ( obj == null ) 
			throw new Exception("Error!  Could not find " + namedProperty + " in " + PROPERTIES_FILE );
		
		return obj.toString();
	}
	
	private boolean isSetToTrue(String namedProperty)
	{
		Object obj = props.get( namedProperty );
		
		if ( obj == null ) 
			return false;
		
		if ( obj.toString().equalsIgnoreCase( TRUE ) || obj.toString().equalsIgnoreCase( YES ) ) 
			return true;
		
		return false;
	}
		
	private ConfigReader() throws Exception
	{
		Object o = new Object();
		
		InputStream in = o.getClass().getClassLoader().getSystemResourceAsStream( PROPERTIES_FILE );
		
		if ( in == null )
			throw new Exception("Error!  Could not find " + PROPERTIES_FILE + " anywhere on the current classpath");		

		props = new Properties();
		props.load( in );
		
	}
	
	public static int getXDim() throws Exception
	{
		return Integer.parseInt(getConfigReader().getAProperty(X_DIM));
	}
	
	public static int getYDim() throws Exception
	{
		return Integer.parseInt(getConfigReader().getAProperty(Y_DIM));
	}
	
	public static String getAmprnaDir() throws Exception
	{
		return getConfigReader().getAProperty(AMPRNA_DIR);
	}
	
	public static String getSequenceDir() throws Exception
	{
		return getConfigReader().getAProperty(SEQUENCE_DIR);
	}
	
	public static String getPermutation2OutDir() throws Exception
	{
		return getConfigReader().getAProperty(PERMUTATION_2_OUT_DIR);
	}
	
	public static String getBaselineChipNames() throws Exception
	{
		return getConfigReader().getAProperty(BASELINE_CHIP_NAMES);
	}
	
	public static String getExperimentChipNames() throws Exception
	{
		return getConfigReader().getAProperty(EXPERIMENT_CHIP_NAMES);
	}
	
	
	public static String getAffyAnalysisOutputDir() throws Exception
	{
	    return getConfigReader().getAProperty(AFFY_ANALYSIS_OUTPUT_DIR);
	}
	
	public static String getFdrAnalysisOutputDir() throws Exception
	{
		return getConfigReader().getAProperty(FDR_ANALYSIS_OUTPUT_DIR);
	}

	public static String getDreauDirectory() throws Exception
	{
	    return getConfigReader().getAProperty(DREAU_DIRECTORY);
	}
	
	public static String getChristineCel() throws Exception
	{
		return getConfigReader().getAProperty(CHRISTINE_CEL);
	}
	
	public static String getProstateDir() throws Exception
	{
		return getConfigReader().getAProperty(PROSTATE_DIR);
	}
	
	public static int getExpectedNumberOfProbe() throws Exception
	{
		return Integer.parseInt(getConfigReader().getAProperty(EXPECTED_NUMBER_PROBES));
	}
	
	public static String getU95LatinSquareDir() throws Exception
	{
		return getConfigReader().getAProperty(U95_LATIN_SQUARE_DIR);
	}
	
	public static boolean getVerboseConsole() throws Exception
	{
	    return getConfigReader().isSetToTrue(VERBOSE_CONSOLE);
	}
	
	public static String getLatinSquareDirectory() throws Exception
	{
	    return getConfigReader().getAProperty(LATIN_SQUARE_DIRECTORY);
	}
	
	public static String getNaefDir() throws Exception
	{
		return getConfigReader().getAProperty(NAEF_DIRECTORY);
	}
	
	public static String getChoeDir() throws Exception
	{
		return getConfigReader().getAProperty(CHOE_DIR);
	}
	
	public static String getAldrichDir() throws Exception
	{
		return getConfigReader().getAProperty(ALDRICH_DIR);
	}
	
	public static String getTimDir() throws Exception
	{
		return getConfigReader().getAProperty(TIM_DIR);
	}
	
	public static String getBlastDirectory() throws Exception
	{
	    return getConfigReader().getAProperty(BLAST_DIRECTORY);
	}
	
	public static String getRDirectory() throws Exception
	{
		return getConfigReader().getAProperty( R_DIRECTORY);
	}
	
	public static String getJulieCelDir() throws Exception
	{
		return getConfigReader().getAProperty(JULIE_CEL_DIR);
	}
	
	public static String getNormalizedAffyCellDir() throws Exception
	{
		return getConfigReader().getAProperty(NORMALIZED_AFFY_CEL_DIR);
	}
	
	public static String getPathToLog2Data() throws Exception
	{
		return getConfigReader().getAProperty(PATH_TO_LOG_2_DATA);
	}
	
	public static String getCDFFile() throws Exception
	{
	    return getConfigReader().getAProperty(CDF_FILE);
	}
	
	public static String getExperimentCelsDirectory() throws Exception
	{
	    return getConfigReader().getAProperty(EXPERIMENT_CELS_DIRECTORY);
	}
	
	public static String getBaselineCelsDirectory() throws Exception
	{
	    return getConfigReader().getAProperty(BASELINE_CELS_DIRECTORY);
	}
	
	public static String getPINCDirectory() throws Exception
	{
	    return getConfigReader().getAProperty(PINC_DIRECTORY);
	}
	
	
	
	private static synchronized ConfigReader getConfigReader() throws Exception
	{
		if ( configReader == null ) 
		{
			configReader = new ConfigReader();
		}
		
		return configReader;
	}
}
